import type { Returns } from '@traversable/registry';
import { symbol } from '@traversable/registry';
import type { MinItems } from './items.js';
import type { RequiredKeys } from './properties.js';
import * as Spec from './specification.js';
export type { Unary, Free, Nullary, } from './specification.js';
export { is, RAW, } from './specification.js';
export { Functor, fold, unfold, } from './functor.js';
export type { RequiredKeys } from './properties.js';
export { isRequired, property, wrapOptional, } from './properties.js';
export type { MinItems } from './items.js';
export { minItems } from './items.js';
export type { JsonSchema, LowerBound, };
export { SymbolJsonSchema, VoidJsonSchema, UndefinedJsonSchema, BigIntJsonSchema, NeverJsonSchema, InlineJsonSchema, UnknownJsonSchema, AnyJsonSchema, NullJsonSchema, BooleanJsonSchema, IntegerJsonSchema, NumberJsonSchema, StringJsonSchema, EqJsonSchema, OptionalJsonSchema, ArrayJsonSchema, RecordJsonSchema, TupleJsonSchema, UnionJsonSchema, IntersectJsonSchema, ObjectJsonSchema, };
type JsonSchema<T = never> = [T] extends [never] ? Spec.JsonSchema : Spec.Unary<T>;
interface LowerBound {
    jsonSchema(): JsonSchema | undefined;
}
interface NeverJsonSchema {
    jsonSchema(): undefined;
}
declare const NeverJsonSchema: NeverJsonSchema;
interface VoidJsonSchema {
    jsonSchema(): undefined;
}
declare const VoidJsonSchema: VoidJsonSchema;
interface UndefinedJsonSchema {
    jsonSchema(): undefined;
}
declare const UndefinedJsonSchema: UndefinedJsonSchema;
interface SymbolJsonSchema {
    jsonSchema(): undefined;
}
declare const SymbolJsonSchema: SymbolJsonSchema;
interface BigIntJsonSchema {
    jsonSchema(): undefined;
}
declare const BigIntJsonSchema: BigIntJsonSchema;
interface InlineJsonSchema<S> {
    jsonSchema: () => void;
}
declare function InlineJsonSchema<S>(_: S): InlineJsonSchema<S>;
declare const AnyJsonSchema: AnyJsonSchema;
interface AnyJsonSchema {
    jsonSchema(): typeof Spec.RAW.any;
}
declare const UnknownJsonSchema: UnknownJsonSchema;
interface UnknownJsonSchema {
    jsonSchema(): typeof Spec.RAW.any;
}
declare const NullJsonSchema: NullJsonSchema;
interface NullJsonSchema {
    jsonSchema(): typeof Spec.RAW.null;
}
declare const BooleanJsonSchema: BooleanJsonSchema;
interface BooleanJsonSchema {
    jsonSchema(): typeof Spec.RAW.boolean;
}
declare const IntegerJsonSchema: IntegerJsonSchema;
interface IntegerJsonSchema {
    jsonSchema(): typeof Spec.RAW.integer;
}
declare const NumberJsonSchema: NumberJsonSchema;
interface NumberJsonSchema {
    jsonSchema(): typeof Spec.RAW.number;
}
declare const StringJsonSchema: StringJsonSchema;
interface StringJsonSchema {
    jsonSchema(): typeof Spec.RAW.string;
}
interface EqJsonSchema<S> {
    jsonSchema(): {
        const: S;
    };
}
declare function EqJsonSchema<V>(value: V): EqJsonSchema<V>;
interface OptionalJsonSchema<S> {
    jsonSchema: {
        [symbol.optional]: number;
        (): Returns<S['jsonSchema' & keyof S]>;
    };
}
declare function OptionalJsonSchema<S>(x: S): OptionalJsonSchema<S>;
interface ArrayJsonSchema<S> {
    jsonSchema(): {
        type: 'array';
        items: Returns<S['jsonSchema' & keyof S]>;
    };
}
declare function ArrayJsonSchema<S>(schema: S): ArrayJsonSchema<S>;
interface RecordJsonSchema<S> {
    jsonSchema(): {
        type: 'object';
        additionalProperties: Returns<S['jsonSchema' & keyof S]>;
    };
}
declare function RecordJsonSchema<S>(schema: S): RecordJsonSchema<S>;
interface UnionJsonSchema<S> {
    jsonSchema(): {
        anyOf: {
            [I in keyof S]: Returns<S[I]['jsonSchema' & keyof S[I]]>;
        };
    };
}
declare function UnionJsonSchema<S extends readonly unknown[]>(schemas: S): UnionJsonSchema<S>;
interface IntersectJsonSchema<S> {
    jsonSchema(): {
        allOf: {
            [I in keyof S]: Returns<S[I]['jsonSchema' & keyof S[I]]>;
        };
    };
}
declare function IntersectJsonSchema<S extends readonly unknown[]>(schemas: S): IntersectJsonSchema<S>;
interface TupleJsonSchema<S extends readonly unknown[]> {
    jsonSchema(): {
        type: 'array';
        items: {
            [I in keyof S]: Returns<S[I]['jsonSchema' & keyof S[I]]>;
        };
        additionalItems: false;
        minItems: MinItems<S>;
        maxItems: S['length' & keyof S];
    };
}
declare function TupleJsonSchema<S extends readonly unknown[]>(schemas: readonly [...S]): TupleJsonSchema<S>;
interface ObjectJsonSchema<S, KS extends RequiredKeys<S> = RequiredKeys<S>> {
    jsonSchema(): {
        type: 'object';
        required: {
            [I in keyof KS]: KS[I] & string;
        };
        properties: {
            [K in keyof S]: Returns<S[K]['jsonSchema' & keyof S[K]]>;
        };
    };
}
declare function ObjectJsonSchema<S extends {
    [x: string]: unknown;
}>(schemas: S): ObjectJsonSchema<S>;
//# sourceMappingURL=jsonSchema.d.ts.map