import { fn, symbol } from '@traversable/registry';
import { minItems } from './items.js';
import { getSchema, isRequired, property, wrapOptional } from './properties.js';
import * as Spec from './specification.js';
export { is, RAW } from './specification.js';
export { Functor, fold, unfold } from './functor.js';
export { isRequired, property, wrapOptional } from './properties.js';
export { minItems } from './items.js';
export { /* no JSON Schema representation */
SymbolJsonSchema, VoidJsonSchema, UndefinedJsonSchema, BigIntJsonSchema, NeverJsonSchema, InlineJsonSchema, /* data types */
UnknownJsonSchema, AnyJsonSchema, NullJsonSchema, BooleanJsonSchema, IntegerJsonSchema, NumberJsonSchema, StringJsonSchema, EqJsonSchema, OptionalJsonSchema, ArrayJsonSchema, RecordJsonSchema, TupleJsonSchema, UnionJsonSchema, IntersectJsonSchema, ObjectJsonSchema };
/** @internal */
const Object_keys = globalThis.Object.keys;
function applyTupleOptionality(xs, {
  min,
  max
}) {
  return min === max ? xs.map(getSchema) : [...xs.slice(0, min).map(getSchema), ...xs.slice(min).map(getSchema)];
}
const NeverJsonSchema = {
  jsonSchema() {
    return void 0;
  }
};
const VoidJsonSchema = {
  jsonSchema() {
    return void 0;
  }
};
const UndefinedJsonSchema = {
  jsonSchema() {
    return void 0;
  }
};
const SymbolJsonSchema = {
  jsonSchema() {
    return void 0;
  }
};
const BigIntJsonSchema = {
  jsonSchema() {
    return void 0;
  }
};
function InlineJsonSchema(_) {
  return {
    jsonSchema: () => void 0
  };
}
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
const AnyJsonSchema = {
  jsonSchema() {
    return Spec.RAW.any;
  }
};
const UnknownJsonSchema = {
  jsonSchema() {
    return Spec.RAW.any;
  }
};
const NullJsonSchema = {
  jsonSchema() {
    return Spec.RAW.null;
  }
};
const BooleanJsonSchema = {
  jsonSchema() {
    return Spec.RAW.boolean;
  }
};
const IntegerJsonSchema = {
  jsonSchema() {
    return Spec.RAW.integer;
  }
};
const NumberJsonSchema = {
  jsonSchema() {
    return Spec.RAW.number;
  }
};
const StringJsonSchema = {
  jsonSchema() {
    return Spec.RAW.string;
  }
};
function EqJsonSchema(value) {
  return {
    jsonSchema() {
      return {
        const: value
      };
    }
  };
}
function OptionalJsonSchema(x) {
  function jsonSchema() {
    return getSchema(x);
  }
  jsonSchema[symbol.optional] = wrapOptional(x);
  return {
    jsonSchema
  };
}
function ArrayJsonSchema(x) {
  return {
    jsonSchema() {
      return {
        type: 'array',
        items: getSchema(x)
      };
    }
  };
}
function RecordJsonSchema(x) {
  return {
    jsonSchema() {
      return {
        type: 'object',
        additionalProperties: getSchema(x)
      };
    }
  };
}
function UnionJsonSchema(xs) {
  return {
    jsonSchema() {
      return {
        anyOf: xs.map(getSchema)
      };
    }
  };
}
function IntersectJsonSchema(xs) {
  return {
    jsonSchema() {
      return {
        allOf: xs.map(getSchema)
      };
    }
  };
}
function TupleJsonSchema(xs) {
  const min = minItems(xs);
  const max = xs.length;
  return {
    jsonSchema() {
      return {
        type: 'array',
        additionalItems: false,
        items: applyTupleOptionality(xs, {
          min,
          max
        }),
        minItems: min,
        maxItems: max
      };
    }
  };
}
function ObjectJsonSchema(xs) {
  const required = Object_keys(xs).filter(isRequired(xs));
  return {
    jsonSchema() {
      return {
        type: 'object',
        required,
        properties: fn.map(xs, property(required))
      };
    }
  };
}
//# sourceMappingURL=jsonSchema.js.map